import { and, eq, gt, sql } from "drizzle-orm";
import { db } from "./db";
import { accounts, receiptItems, receipts } from "./schema";

export interface AccountSummary {
    id: number,
    name: string,
    balance: bigint,
    currency: string,
    lastReceipt?: Date,
}

export async function getUserAccountSummary(userId: number) {
    return (await db
        .select({
            id: accounts.id,
            name: accounts.name,
            currency: accounts.currency,
            balance: sql<string>`${accounts.startingBalance} + COALESCE(sum(${receiptItems.amount}), 0)`,
            lastReceipt: sql<Date | null>`max(${receipts.date})`,
        })
        .from(accounts)
        .where(eq(accounts.ownerId, userId))
        .leftJoin(receipts, and(eq(receipts.accountFromId, accounts.id), gt(receipts.date, accounts.startingDate)))
        .leftJoin(receiptItems, eq(receiptItems.receiptId, receipts.id))
        .groupBy(accounts.id)).map((row) => ({
            ...row,
            balance: BigInt(row.balance),
            lastReceipt: row.lastReceipt ?? undefined,
        } satisfies AccountSummary))
}
