import { eq } from "drizzle-orm";
import { db } from "./db";
import { users } from "./schema";
import type { RequestEvent } from "../../routes/$types";
import { redirect } from "@sveltejs/kit";

export interface User {
	id: number;
	username: string;
	displayName?: string;
}

export async function getUserFromUsername(
	username: string,
): Promise<User | null> {
	const user = await db.query.users.findFirst({
		columns: {
			id: true,
			username: true,
			displayName: true,
		},
		where: eq(users.username, username),
	});

	if (!user) {
		return null;
	}

	return {
		...user,
		displayName: user.displayName || undefined,
	};
}

export async function getUserPasswordHash(
	userId: number,
): Promise<string | null> {
	const user = await db.query.users.findFirst({
		columns: {
			passwordHash: true,
		},
		where: eq(users.id, userId),
	});

	if (user === undefined) {
		throw new Error("Invalid user ID");
	}

	return user.passwordHash;
}

export function ensureUser(
	event: RequestEvent,
	defaultAction: (() => never) | undefined = undefined,
): User {
	const user = event.locals.user;
	if (user !== null) {
		return user;
	}

	if (defaultAction === undefined) {
		redirect(303, "/login");
	}

    return defaultAction();
}
